/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import java.io.StringReader;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.clientserverapi.jbi.messaging.registry.exception.RegistryException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.servicedesc.JBIServiceEndpointImpl;
import org.ow2.petals.microkernel.api.jbi.servicedesc.Location;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.registry.overlay.api.RegistryOverlayAPI;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;
import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.Transformers;

/**
 * An implementation of {@link PetalsServiceEndpoint} to be store in the Petals Registry Overlay, loading the description
 * lazily.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayEndpoint extends JBIServiceEndpointImpl {

    private static final long serialVersionUID = -6364121452745414838L;

    private final transient LoggingUtil log = new LoggingUtil(
            java.util.logging.Logger.getLogger(EndpointRegistry.COMPONENT_LOGGER_NAME));

    private transient RegistryOverlayAPI registryOverlay;

    public RegistryOverlayEndpoint(final PetalsServiceEndpoint serviceEndpoint, final RegistryOverlayAPI registryOverlay) {
        super();
        this.registryOverlay = registryOverlay;
        this.setEndpointName(serviceEndpoint.getEndpointName());
        this.setServiceName(serviceEndpoint.getServiceName());
        this.setInterfacesName(serviceEndpoint.getInterfaces());
        this.setDescription(null);
        this.setLocation(serviceEndpoint.getLocation());
        this.setType(serviceEndpoint.getType());
    }

    public RegistryOverlayEndpoint(final String endpointName, final QName serviceName, final List<QName> interfaces,
            final Location location, final RegistryOverlayAPI registryOverlay) {
        super();
        this.registryOverlay = registryOverlay;
        this.setEndpointName(endpointName);
        this.setServiceName(serviceName);
        this.setInterfacesName(interfaces);
        this.setDescription(null);
        this.setLocation(location);
        this.setType(EndpointType.INTERNAL);
    }

    public void setRegistryOverlay(final RegistryOverlayAPI registryOverlay) {
        this.registryOverlay = registryOverlay;
    }

    @Override
    public Document getDescription() {
        synchronized (this) {
            final Document description = super.getDescription();
            if (description != null) {
                return description;
            } else {
                try {
                    this.retrieveDescription();
                    return super.getDescription();
                } catch (final RegistryException e) {
                    this.log.warning("An error occurs retrieving the endpoint description", e);
                    return null;
                }
            }
        }
    }

    /**
     * Set the WSDL into the service endpoint
     */
    private void retrieveDescription() throws RegistryException {

        final String wsdlStr = this.registryOverlay.retrieveDescription(this.getEndpointName());
        if (wsdlStr != null) {
            final Transformer transformer = Transformers.takeTransformer();
            final StringReader wsdlReader = new StringReader(wsdlStr);
            final StreamSource xmlSource = new StreamSource(wsdlReader);
            final Document document = DocumentBuilders.newDocument();
            final DOMResult domResult = new DOMResult(document);
            try {
                transformer.transform(xmlSource, domResult);
                this.setDescription(document);
            } catch (final TransformerException e) {
                throw new RegistryException(e);
            } finally {
                wsdlReader.close();
                Transformers.releaseTransformer(transformer);
            }
        }

    }

}
