/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.util.Arrays;

import javax.xml.namespace.QName;

import org.easymock.EasyMock;
import org.junit.Test;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointDoesNotExistsException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.servicedesc.Location;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * Unit tests of {@link RegistryOverlayImpl#deactivateEndpoint(String, QName)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_DeactivateEndpoint_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A namespace name
     */
    private final static String A_NAMESPACE = "http://petals.ow2.org/unit-tests";

    /**
     * A service name
     */
    private final static QName SERVICE_NAME_1 = new QName(A_NAMESPACE, "service-1");

    /**
     * A interface name
     */
    private final static QName INTERFACE_NAME_1 = new QName(A_NAMESPACE, "interface-1");

    /**
     * A endpoint name
     */
    private final static String ENDPOINT_NAME_1;

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_1 = "container-1";

    static {
        final QualifiedUUIDGenerator uuidGenerator = new QualifiedUUIDGenerator("petals");
        ENDPOINT_NAME_1 = uuidGenerator.getNewID();
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#deactivateEndpoint(String, QName)}:
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>deactivation of an endpoint previously correctly activated.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs, the endpoint is correctly deactivated</li>
     * <li>no more endpoint is retrieve using {@link EndpointRegistry#getEndpoint(QName, String)}</li>
     * <li>no more endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForService(QName, LinkType)}</li>
     * <li>no more endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForInterface(QName, LinkType)}
     * </li>
     * </ol>
     * </p>
     */
    @Test
    public void testToDeactivateOneValidEndpoint() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_DeactivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_testToDeactivateOneValidEndpoint", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            final PetalsServiceEndpoint serviceEndpointMock = EasyMock.createMock(PetalsServiceEndpoint.class);
            EasyMock.expect(serviceEndpointMock.getLocation()).andReturn(new Location()).anyTimes();
            EasyMock.replay(serviceEndpointMock);

            // Add the endpoint
            registryOverlayImpl.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, serviceEndpointMock, false);

            // Retrieve the endpoint
            final PetalsServiceEndpoint retrievedEndpoint = registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1);
            assertNotNull(retrievedEndpoint);
            assertEquals(retrievedEndpoint.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedEndpoint.getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the service
            final PetalsServiceEndpoint[] svcEpForSvc = registryOverlayImpl.getInternalEndpointsForService(SERVICE_NAME_1,
                    LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForSvc);
            assertEquals("No endpoint retrieved for service", svcEpForSvc.length, 1);
            assertEquals(svcEpForSvc[0].getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the interface
            final PetalsServiceEndpoint[] svcEpForItf = registryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForItf);
            assertEquals("No endpoint retrieved for interface", svcEpForItf.length, 1);
            assertEquals(svcEpForItf[0].getEndpointName(), ENDPOINT_NAME_1);

            // Deactivate the endpoint
            registryOverlayImpl.deactivateEndpoint(ENDPOINT_NAME_1, SERVICE_NAME_1);

            // Retrieve the endpoint
            assertNull("Endpoint retrieved", registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1));

            // Check that no more endpoint is registered as implementing the service
            final PetalsServiceEndpoint[] noMoreSvcEpForSvc = registryOverlayImpl.getInternalEndpointsForService(
                    SERVICE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", noMoreSvcEpForSvc);
            assertEquals("Endpoint retrieved for service", noMoreSvcEpForSvc.length, 0);

            // Check that no more endpoint is registered as implementing the interface
            final PetalsServiceEndpoint[] noMoreSvcEpForItf = registryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", noMoreSvcEpForItf);
            assertEquals("Endpoint retrieved for interface", noMoreSvcEpForItf.length, 0);

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#deactivateEndpoint(String, QName)}:
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>deactivation of an endpoint not previously activated.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>an error occurs, the endpoint does not exist</li>
     * </ol>
     * </p>
     */
    @Test(expected = EndpointDoesNotExistsException.class)
    public void testToDeactivateAnNotExistinglidEndpoint() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_DeactivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_testToDeactivateOneValidEndpoint", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            // Deactivate the endpoint
            registryOverlayImpl.deactivateEndpoint(ENDPOINT_NAME_1, SERVICE_NAME_1);

        } finally {
            registryOverlayImpl.stop();
        }
    }

}
