/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.net.URL;
import java.util.Arrays;

import javax.xml.namespace.QName;

import org.junit.Test;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlReaders;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlWriters;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.petals.microkernel.api.jbi.servicedesc.JBIServiceEndpointImpl;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint.EndpointType;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * Unit tests of {@link RegistryOverlayImpl#getEndpoint(QName, String)}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_GetEndpoint_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A namespace name
     */
    private final static String A_NAMESPACE = "http://petals.ow2.org/unit-tests";

    /**
     * A service name
     */
    private final static QName SERVICE_NAME_1 = new QName(A_NAMESPACE, "service-1");

    /**
     * A interface name
     */
    private final static QName INTERFACE_NAME_1 = new QName(A_NAMESPACE, "interface-1");

    /**
     * A local container name
     */
    private final static String LOCAL_CONTAINER_NAME = "local-container";

    /**
     * An internal endpoint name on a local container
     */
    private final static String LOCAL_INTERNAL_ENDPOINT_NAME_1;

    static {
        final QualifiedUUIDGenerator uuidGenerator = new QualifiedUUIDGenerator("petals");
        LOCAL_INTERNAL_ENDPOINT_NAME_1 = uuidGenerator.getNewID();
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#getEndpoint(QName, String)} :
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>an internal endpoints is registered on a local container.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs</li>
     * <li>the endpoint is retrieved with the right attributes</li>
     * </ol>
     * </p>
     */
    @Test
    public void testToGetAnEndpoint() throws Exception {

        final String topologyName = this.getClass().getSimpleName() + "_testToGetAnEndpoint";
        final RegistryOverlayImpl localRegistryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(localRegistryOverlayImpl, "configurationService",
                RegistryOverlayImpl_GetEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), topologyName, LOCAL_CONTAINER_NAME));

        try {
            localRegistryOverlayImpl.start();

            final JBIServiceEndpointImpl localInternalServiceEndpointMock_1 = new JBIServiceEndpointImpl();
            localInternalServiceEndpointMock_1.setEndpointName(LOCAL_INTERNAL_ENDPOINT_NAME_1);
            localInternalServiceEndpointMock_1.setServiceName(SERVICE_NAME_1);
            localInternalServiceEndpointMock_1.setInterfacesName(Arrays.asList(new QName[] { INTERFACE_NAME_1 }));
            localInternalServiceEndpointMock_1.setType(EndpointType.INTERNAL);

            // Read the wsdl
            final URL wsdlUrl = Thread.currentThread().getContextClassLoader().getResource("echo.wsdl");
            assertNotNull("WSDL not found", wsdlUrl);
            final WSDL4ComplexWsdlReader wsdl4ComplexWsdlReader = WSDL4ComplexWsdlReaders.takeWSDL4ComplexWsdlReader();
            try {
                final Description wsdlDescription = wsdl4ComplexWsdlReader.read(wsdlUrl);
                wsdlDescription.addImportedDocumentsInWsdl();
                final WSDL4ComplexWsdlWriter wsdl4ComplexWsdlWriter = WSDL4ComplexWsdlWriters
                        .takeWSDL4ComplexWsdlWriter();
                try {
                    final Document document = wsdl4ComplexWsdlWriter.getDocument(wsdlDescription);

                    // Add endpoints
                    localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_1, LOCAL_INTERNAL_ENDPOINT_NAME_1,
                            Arrays.asList(new QName[] { INTERFACE_NAME_1 }), document,
                            localInternalServiceEndpointMock_1, false);

                } finally {
                    WSDL4ComplexWsdlWriters.releaseWSDL4ComplexWsdlWriter(wsdl4ComplexWsdlWriter);
                }

            } finally {
                WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(wsdl4ComplexWsdlReader);
            }

            // Check that endpoint is registered
            final PetalsServiceEndpoint registeredEp = localRegistryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    LOCAL_INTERNAL_ENDPOINT_NAME_1);
            assertNotNull("Null list returned", registeredEp);
            assertEquals("Unexpected service name", SERVICE_NAME_1, registeredEp.getServiceName());
            assertEquals("Unexpected endpoint name", LOCAL_INTERNAL_ENDPOINT_NAME_1, registeredEp.getEndpointName());
            final Document retrievedDocument = registeredEp.getDescription();
            assertNotNull("WSDL description is null", retrievedDocument);
            final WSDL4ComplexWsdlReader anotherComplexWsdlReader = WSDL4ComplexWsdlReaders
                    .takeWSDL4ComplexWsdlReader();
            try {
                final Description wsdlDescription = anotherComplexWsdlReader.read(retrievedDocument);
                assertNotNull("EasyWSDL description is null", wsdlDescription);

            } finally {
                WSDL4ComplexWsdlReaders.releaseWSDL4ComplexWsdlReader(anotherComplexWsdlReader);
            }

        } finally {
            localRegistryOverlayImpl.stop();
        }
    }

}
